#include "complex.h"
#include <math.h>
using namespace std;

// ---------------------------------------------------------------------------
// Klasse zum Rechnen mit komplexen Zahlen
// ---------------------------------------------------------------------------
Complex::Complex()                         { Set(0,    0);    }
Complex::Complex(double real)              { Set(real, 0);    }
Complex::Complex(double real, double imag) { Set(real, imag); }

Complex::~Complex() { /* cout << "Destruktor..." << endl; */ }

double Complex::Real() { return re; }
double Complex::Imag() { return im; }

void Complex::Set(double real, double imag)
{
    re = real; im = imag;
}

double Complex::Abs()
{
    return sqrt(re * re + im * im);
}

double Complex::Arg()
{
    if(re > 0)
        return atan(im / re);
    else if(re == 0 && im > 0)
        return M_PI / 2.0;
    else if(re == 0 && im < 0)
        return -M_PI / 2.0;
    else if(re < 0 && im > 0)
        return atan(im / re) + M_PI;
    else if(re < 0 && im < 0)
        return atan(im / re) - M_PI;
    else
        return 0;
}

// ---------------------------------------------------------------------------
// Freie Funktionen bzw. Operatoren zum Rechnen mit komplexen Zahlen
// ---------------------------------------------------------------------------
Complex operator+ (Complex a, Complex b)
{
    Complex sum;
    sum.Set(a.Real() + b.Real(), a.Imag() + b.Imag());
    return sum;
}

Complex operator- (Complex a, Complex b)
{
    Complex diff;
    diff.Set(a.Real() - b.Real(), a.Imag() - b.Imag());
    return diff;
}

Complex operator* (Complex a, Complex b)
{
    Complex result(
                a.Real() * b.Real() - a.Imag() * b.Imag(),
                a.Real() * b.Imag() + a.Imag() * b.Real());
    return result;
}

Complex operator/ (Complex a, Complex b)
{
    double denom = b.Real() * b.Real() + b.Imag() * b.Imag();
    double real1 = a.Real() * b.Real() + a.Imag() * b.Imag();
    double imag1 = a.Imag() * b.Real() - a.Real() * b.Imag();

    if(denom == 0)
    {
        cout << "Klasse Complex: Division durch null!" << endl;
        Complex null;
        return null;
    }
    else
    {
        Complex result(real1 / denom, imag1 / denom);
        return result;
    }
}

ostream& operator<< (ostream& strm, Complex c)
{
    strm << "(" << c.Real();
    if(c.Imag() >= 0)
        strm << " + " << c.Imag() << "i)";
    else
        strm << " - " << -c.Imag() << "i)";
    return strm;
}

istream& operator>> (istream& strm, Complex& c)
{
    char trennzeichen;
    double r, i;
    strm >> r >> trennzeichen;
    if(trennzeichen != ',' && trennzeichen != ';')
        strm.putback(trennzeichen);
    strm >> i;
    c.Set(r, i);
    return strm;
}

Complex exp(Complex z)
{
    Complex result;
    result.Set(exp(z.Real()) * cos(z.Imag()),
               exp(z.Real()) * sin(z.Imag()));
    return result;
}
