#define _USE_MATH_DEFINES
#include <iostream>
#include <vector>
#include <math.h>
#include <time.h>
#include "chart.h"
#include "complex.h"
using namespace std;
typedef vector<Complex> cplx_vec;

// ---------------------------------------------------------------------------
// FFT für (komplexen) Vektor a berechnen.
// Achtung: Die Anzahl der Elemente im Vektor a muss eine Zweierpotenz sein!
// ---------------------------------------------------------------------------
cplx_vec fft(cplx_vec a)
{
    int k = 0, n = a.size();
    if(n == 1) return a;

    Complex tmp(0, -2 * M_PI / n);
    Complex omega = 1, omega_n = exp(tmp);
    cplx_vec a_i, a_ii;

    while(k < n)
    {
        a_i.push_back(a[k++]);
        a_ii.push_back(a[k++]);
    }

    auto y_i = fft(a_i), y_ii = fft(a_ii);
    cplx_vec y(n);

    for(k = 0; k < n / 2; k++)
    {
        y[k] = y_i[k] + omega * y_ii[k];
        y[k + n / 2] = y_i[k] - omega * y_ii[k];
        omega = omega * omega_n;
    }

    return y;
}

// ---------------------------------------------------------------------------
// FFT-Demoprogramm: Siehe MATLAB-Dokumentation zum fft()-Befehl...
// ---------------------------------------------------------------------------
int main(void)
{
    srand(time(0));
    double Fs = 1000.0;  // Sampling frequency
    double T = 1 / Fs;   // Sampling period
    int i, L = 1024;     // Length of signal (ACHTUNG: ZWEIERPOTENZ)

    /*
        S = 0.7*sin(2*pi*50*t) + sin(2*pi*120*t);
        X = S + 2*randn(size(t));
    */

    cplx_vec X;
    for(i = 0; i < L; ++i)
    {
        auto S = 0.7 * sin(2.0 * M_PI * 50.0 * i * T);
        S += sin(2.0 * M_PI * 120.0 * i * T);
        S += 2.0 * rand() / RAND_MAX - 1.0;
        chart_lineto(1000.0 * i * T, S, CHART_RED);
        X.push_back(S);
    }

    /*
        plot(1000*t(1:50),X(1:50))
        title('Signal Corrupted with Zero-Mean Random Noise')
        xlabel('t (milliseconds)')
        ylabel('X(t)')
    */

    chart_axis_fmt("%g ms", "%g");
    chart_axis_xlimit(0, 50);
    chart_show(CHART_YZOOM_OFF | CHART_XLIMIT, "Signal Corrupted with Noise");
    chart_clear();

    /*
        Y = fft(X);
        P2 = abs(Y/L);
        P1 = P2(1:L/2+1);
        P1(2:end-1) = 2*P1(2:end-1);
        f = Fs*(0:(L/2))/L;
        plot(f,P1)
        title('Single-Sided Amplitude Spectrum of X(t)')
        xlabel('f (Hz)')
        ylabel('|P1(f)|')
    */

    auto Y = fft(X);
    for(i = 0, Y[0] = Y[0] / 2; i < L / 2; ++i)
    {
        auto P = 2.0 * Y[i].Abs() / L;
        chart_lineto(i * Fs / L, P, CHART_BLUE);
    }

    chart_axis_fmt("%g Hz", "%g");
    chart_show(CHART_YZOOM_OFF, "Single-Sided Amplitude Spectrum");
}
